@@ -1,4 +1,4 @@
-#!/usr/bin/env python
+#!/usr/bin/env python3
 # encoding: utf-8
 """
 run_tests.py
@@ -25,6 +25,7 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 import xpdeint.Python24Support
 
 import os
+import io
 import re
 import sys
 import getopt
@@ -33,6 +34,7 @@ import hashlib
 import unittest
 import subprocess
 
+#from io import open
 from xml.dom import minidom
 import xpdeint.minidom_extras
 from xpdeint import CodeParser
@@ -58,10 +60,20 @@ def pass_nan_test(array1, array2):
 
 def array_approx_equal(array1, array2, absTol, relTol):
   """Return `True` if all of (`array1` - `array2`) <= `absTol` or (`array1` - `array2`) <= `relTol` * `array2`"""
-  diff = array1-array2
+  absdiff = array1 - array2
+  abssum = numpy.abs(array1 + array2)
+
   # NaN values would fail this test. So we have to exclude them. But only exclude them if array2 (the expected results)
-  # have a NaN
-  return numpy.logical_or(numpy.logical_or(numpy.abs(diff) <= 0.5 * relTol * (numpy.abs(array2) + numpy.abs(array1)), numpy.abs(diff) <= absTol), numpy.isnan(array2)).all()
+  # have NaNs in the same places
+
+  if numpy.isnan(array1.all()) == numpy.isnan(array2.all()):
+    # There are NaNs in the answer, but they are in the same place for both expected and current current simulation results.
+    # This means we can remove them.
+    absdiff = absdiff[~numpy.isnan(array1)]
+    abssum = abssum[~numpy.isnan(array2)]
+
+  return numpy.logical_or(absdiff <= 0.5 * relTol * abssum, absdiff <= absTol).all()
+
 
 def scriptTestingFunction(root, scriptName, testDir, absPath, self):
   if not os.path.exists(testDir):
@@ -91,7 +103,7 @@ def scriptTestingFunction(root, scriptName, testDir, absPath, self):
     if re.search(r'This script requires the python package', message, re.MULTILINE):
       self.skipTest("Skipping test as python package required is not installed")
 
-  self.assert_(returnCode == 0, ("Failed to compile." % locals()) + message)
+  self.assertTrue(returnCode == 0, ("Failed to compile." % locals()) + message)
   
   xmlDocument = minidom.parse(absPath)
   simulationElement = xmlDocument.getChildElementByTagName('simulation')
@@ -103,14 +115,23 @@ def scriptTestingFunction(root, scriptName, testDir, absPath, self):
   # If the source is the same as the last known good, then we don't need to compile or execute the simulation.
   sourceFilePath = os.path.join(testDir, simulationName + '.cc')
   checksumFilePath = os.path.join(testDir, simulationName + '_last_known_good.checksum')
-  sourceContents = file(sourceFilePath).read()
+
+  #sourceContents = file(sourceFilePath).read()
+
+  with io.open(sourceFilePath, 'r') as f:
+    sourceContents = f.read()
+
   h = hashlib.sha1()
-  h.update(sourceContents)
+  h.update(sourceContents.encode('utf-8'))
   currentChecksum = h.hexdigest()
   
   if os.path.exists(checksumFilePath):
-    lastKnownGoodChecksum = file(checksumFilePath).read()
-    
+    #lastKnownGoodChecksum = file(checksumFilePath).read()
+
+    with io.open(checksumFilePath, 'r') as f:
+      lastKnownGoodChecksum = f.read()
+
+
     if lastKnownGoodChecksum == currentChecksum:
       # The checksums check out, so we don't need to go any further
       return
@@ -148,7 +169,7 @@ def scriptTestingFunction(root, scriptName, testDir, absPath, self):
   (stdout, stderr) = simulationProc.communicate()
   returnCode = simulationProc.wait()
   
-  self.assert_(returnCode == 0, "Failed to execute compiled simulation correctly. Got returnCode %(returnCode)i;\nstdout = %(stdout)s;\nstderr = %(stderr)s\n" % locals())
+  self.assertTrue(returnCode == 0, "Failed to execute compiled simulation correctly. Got returnCode %(returnCode)i;\nstdout = %(stdout)s;\nstderr = %(stderr)s\n" % locals())
   
   # The next thing to check is that the generated data agrees with the expected data to within the set error margins.
   xsilFileElements = testingElement.getChildElementsByTagName('xsil_file', optional=True)
@@ -168,18 +189,20 @@ def scriptTestingFunction(root, scriptName, testDir, absPath, self):
     results = XSILFile(resultsFullPath)
     expectedResultsFullPath = os.path.join(os.path.split(absPath)[0], expectedResultsFile)
     if not os.path.exists(expectedResultsFullPath):
-      print >> sys.stderr, "Expected results file '%(expectedResultsFile)s' missing. Using current. " % locals()
-      
+      sys.stderr.write("Expected results file '%(expectedResultsFile)s' missing. Using current. " % locals())
       # If there are any NaN's in the results, issue a warning.
       for mgNum, o in enumerate(results.xsilObjects):
         for v in o.independentVariables:
           if numpy.isnan(v['array']).any():
-            print >> sys.stderr, "Warning: Coordinate '%s' in moment group %i of file '%s' contains a NaN." % (v['name'], mgNum+1, sourceFile)
+            sys.stderr.write("Warning: Coordinate '%s' in moment group %i of file '%s' contains a NaN." % (v['name'], mgNum+1, sourceFile))
         for v in o.dependentVariables:
           if numpy.isnan(v['array']).any():
-            print >> sys.stderr, "Warning: Dependent variable '%s' in moment group %i of file '%s' contains a NaN." % (v['name'], mgNum+1, sourceFile)
+            sys.stderr.write("Warning: Dependent variable '%s' in moment group %i of file '%s' contains a NaN." % (v['name'], mgNum+1, sourceFile))
       
-      resultsFileContents = file(resultsFullPath).read()
+      #resultsFileContents = file(resultsFullPath).read()
+      with io.open(resultsFullPath, 'r') as f:
+        resultsFileContents = f.read()
+
       
       for xsilObject in results.xsilObjects:
         if hasattr(xsilObject.data, 'filename'):
@@ -191,24 +214,47 @@ def scriptTestingFunction(root, scriptName, testDir, absPath, self):
           shutil.copyfile(os.path.join(testDir, xsilObject.data.filename),
                           os.path.join(os.path.split(absPath)[0], newDataFilename))
       
-      file(expectedResultsFullPath, 'w').write(resultsFileContents)
+      #file(expectedResultsFullPath, 'w').write(resultsFileContents)
+      with io.open(expectedResultsFullPath, 'w') as f:
+        f.write(resultsFileContents)
+
     else:
+      # We enter this path for each xsil file in a testing element, provided a previous
+      # expected result exists that we can can compare with
+      # results is the current run results
       expectedResults = XSILFile(expectedResultsFullPath)
       
-      self.assert_(len(results.xsilObjects) == len(expectedResults.xsilObjects))
+      self.assertTrue(len(results.xsilObjects) == len(expectedResults.xsilObjects))
       
       momentGroupElements = xsilFileElement.getChildElementsByTagName('moment_group', optional=True)
+
+      # If moment groups elements are specfically listed in the testing XML element, make sure the 
+      # number listed corresponds to the number of xsilobjects in the results. 
+      # If they're no listed in the testing block, set up a blank vector for the moment groups
+      # with a number of entry equal to the number of xsilobjects
       if momentGroupElements:
-        self.assert_(len(momentGroupElements) == len(results.xsilObjects))
+        self.assertTrue(len(momentGroupElements) == len(results.xsilObjects))
       else:
         momentGroupElements = [None]*len(results.xsilObjects)
       
+      # In the for loop below, the "enumerate" keyword assigns a loop counter to the variable mgNum
+      # The "zip" keyword creates a tuple from the three listed objects, and sets up an iterator over
+      # them that finishes when the shortest of the 3 lists of elements is exhausted
       for mgNum, (o1, o2, mgElem) in enumerate(zip(results.xsilObjects, expectedResults.xsilObjects, momentGroupElements)):
+        # o1.name is something like "moment_group_3" or "breakpoint"
+        # o1.independentVariables is a list of independent variables. Each element in the list is a dict.
+        #    Each dict has keys like "name", "length", "array". "name" are things like "t", "x", "y" etc.
+        # o1.dependentVariables is a list with the outputs of a single moment group, so things like real and imaginary
+        #    parts of a potential vector. If there are four moments, the list will have four elements. Each element
+        #    is a dictionary with keys "name" and "array". "name" is the moment name. "array" is an array with all
+        #    the data for that moment. 
+
         currentAbsoluteTolerance = absoluteTolerance
         currentRelativeTolerance = relativeTolerance
-        self.assert_(len(o1.independentVariables) == len(o2.independentVariables),
+        self.assertTrue(len(o1.independentVariables) == len(o2.independentVariables),
                      "The number of independent variables in moment group %(mgNum)i doesn't match." % locals())
-        self.assert_(len(o1.dependentVariables) == len(o2.dependentVariables),
+
+        self.assertTrue(len(o1.dependentVariables) == len(o2.dependentVariables),
                      "The number of dependent variables in moment group %(mgNum)i doesn't match." % locals())
         
         if mgElem:
@@ -217,28 +263,63 @@ def scriptTestingFunction(root, scriptName, testDir, absPath, self):
           if mgElem.hasAttribute('relative_tolerance'):
             currentRelativeTolerance = float(mgElem.getAttribute('relative_tolerance'))
         
-        self.assert_(currentAbsoluteTolerance != None and currentRelativeTolerance != None, "An absolute and a relative tolerance must be specified.")
+        self.assertTrue(currentAbsoluteTolerance != None and currentRelativeTolerance != None, "An absolute and a relative tolerance must be specified.")
         
         for v1, v2 in zip(o1.independentVariables, o2.independentVariables):
-          self.assert_(v1['name'] == v2['name'])
-          self.assert_(v1['length'] == v2['length'])
+          self.assertTrue(v1['name'] == v2['name'])
+          self.assertTrue(v1['length'] == v2['length'])
           # These are the coordinates, we just specify a constant absolute and relative tolerance.
           # No-one should need to change these
-          self.assert_(array_approx_equal(v1['array'], v2['array'], 1e-7, 1e-6),
+          self.assertTrue(array_approx_equal(v1['array'], v2['array'], 1e-7, 1e-6),
                       "Coordinate '%s' in moment group %i of file '%s' didn't pass tolerance criteria." % (v1['name'], mgNum+1, sourceFile))
         
+        # Now we have to start being careful. The assert below used to work in Python 2:
+
+        # for v1, v2 in zip(o1.dependentVariables, o2.dependentVariables):
+        #  self.assertTrue(v1['name'] == v2['name'])
+
+        # But in Python 3 the assert can sometimes fail, because dictionary hashing became random. This means iterating over a dict
+        # can give varying element orders per iteration. This in turn means the "first" element of this test run may not
+        # be the same as the "first" element of the saved run we're comparing against. 
+        #
+        # This should always be safe for independent variables (I think) because there's only one per moment group.
+        # But there may be several dependent variables (moments) per moment group, and their dict order when iterated
+        # over may be different to the expected value file's iterated dict order. 
+        # 
+        # So for Python 3 we need to change the code to match up moments by name within the moment group
+        # when comparing, not just go by position order.
+        #
+        # To do this, given a list o1.dependentVariables, and a list o2.dependentVariables we need to reorder 
+        # both lists so they have same order, based on the "name" key of each list element
+
+        o1.dependentVariables.sort(key = lambda DepVar: DepVar['name']) 
+        o2.dependentVariables.sort(key = lambda DepVar: DepVar['name'])
+
         for v1, v2 in zip(o1.dependentVariables, o2.dependentVariables):
-          self.assert_(v1['name'] == v2['name'])
-          self.assert_(pass_nan_test(v1['array'], v2['array']),
+
+          self.assertTrue(v1['name'] == v2['name'])
+          self.assertTrue(pass_nan_test(v1['array'], v2['array']),
                        "Dependent variable '%s' in moment group %i of file '%s' had a NaN where the expected results didn't (or vice-versa)." % (v1['name'], mgNum+1, sourceFile))
-          self.assert_(array_approx_equal(v1['array'], v2['array'], currentAbsoluteTolerance, currentRelativeTolerance),
+          self.assertTrue(array_approx_equal(v1['array'], v2['array'], currentAbsoluteTolerance, currentRelativeTolerance),
                        "Dependent variable '%s' in moment group %i of file '%s' failed to pass tolerance criteria." % (v1['name'], mgNum+1, sourceFile))
   
-  # Test has succeeded, so save our checksum for the source file and copy the source file
-  file(checksumFilePath, 'w').write(currentChecksum)
+  # Following gymnastics are to make the code work for both Python 2 and Python 3
+  try:
+    basestring # If this doesn't throw an error we're using Python 2
+    with io.open(checksumFilePath, 'w') as f:
+      f.write(currentChecksum.decode('utf-8'))
+  except NameError:
+    # Okay, we're using Python 3
+    with io.open(checksumFilePath, 'w') as f:
+      f.write(currentChecksum)
+
+
   
   lastKnownGoodSourcePath = os.path.join(testDir, simulationName + '_last_known_good.cc')
-  file(lastKnownGoodSourcePath, 'w').write(sourceContents)
+  #file(lastKnownGoodSourcePath, 'w').write(sourceContents)
+
+  with io.open(lastKnownGoodSourcePath, 'w') as f:
+    f.write(sourceContents)
 
 def partial(func, *args, **keywords):
   def newfunc(*fargs, **fkeywords):
@@ -256,7 +337,7 @@ def main(argv=None):
   try:
     try:
       opts, args = getopt.getopt(argv[1:], "ho:v", ["help", "output="])
-    except getopt.error, msg:
+    except getopt.error as msg:
       raise Usage(msg)
   
     # option processing
@@ -268,9 +349,9 @@ def main(argv=None):
       if option in ("-o", "--output"):
         output = value
   
-  except Usage, err:
-    print >> sys.stderr, sys.argv[0].split("/")[-1] + ": " + str(err.msg)
-    print >> sys.stderr, "\t for help use --help"
+  except Usage as err:
+    sys.stderr.write(sys.argv[0].split("/")[-1] + ": " + str(err.msg))
+    sys.stderr.write("\t for help use --help")
     return 2
   
   basePath = os.path.dirname(__file__)
@@ -280,7 +361,7 @@ def main(argv=None):
     os.mkdir(resultsPath)
   resultsPath = os.path.abspath(resultsPath)
   
-  print "Saving test results in %(resultsPath)s" % locals()
+  print(("Saving test results in %(resultsPath)s" % locals()))
   
   testsuites = {}
   baseSuiteName = 'testsuite'
@@ -305,6 +386,7 @@ def main(argv=None):
       class ScriptTestCase(unittest.TestCase):
         # Create test functions for each test script using 'scriptTestingFunction'
         # These test function names are of the form 'test_ScriptName'
+
         for scriptName in xmdsTestScripts:
           prefix = os.path.splitext(scriptName)[0]
           absPath = os.path.abspath(os.path.join(root, scriptName))
@@ -337,7 +419,7 @@ def main(argv=None):
       if fullSuiteName in testsuites:
         suitesToRun.append(testsuites[fullSuiteName])
       else:
-        print >> sys.stderr, "Unable to find test '%(suiteName)s'" % locals()
+        sys.stderr.write("Unable to find test '%(suiteName)s'" % locals())
   else:
     suitesToRun.append(testsuites[baseSuiteName])
   suitesToRun.append(unittest.defaultTestLoader.loadTestsFromModule(CodeParser))
@@ -349,3 +431,8 @@ def main(argv=None):
 
 if __name__ == "__main__":
   sys.exit(main())
+
+
+
+
+
