@@ -1,4 +1,4 @@
-#!/usr/bin/env python
+#!/usr/bin/env python3
 # encoding: utf-8
 """
 _TransformMultiplexer.py
@@ -29,6 +29,8 @@ from xpdeint.Vectors.VectorElement import VectorElement
 from xpdeint.ParserException import ParserException
 
 import operator
+import collections
+from functools import reduce
 
 class _TransformMultiplexer (_Feature):
   featureName = 'TransformMultiplexer'
@@ -251,6 +253,9 @@ class _TransformMultiplexer (_Feature):
             )
             results.append(newState)
         return results
+
+      def __lt__(self, other):
+          return self.location < other.location
       
     def pathFromBasisToBasis(start, endID, pathInfo):
       """
@@ -303,7 +308,7 @@ class _TransformMultiplexer (_Feature):
       basesFieldMap[vector.field.name].update(vectorBases)
     
     # Next step: Perform Dijkstra search over the provided transforms to find the optimal transform map.
-    for basesNeeded in basesFieldMap.values():
+    for basesNeeded in list(basesFieldMap.values()):
       targetStates = [BasisState( (0, 0, 0, 0), (basis, BasisState.IN_PLACE), sourceID) for sourceID, basis in enumerate(basesNeeded)]
       targetLocations = [state.location for state in targetStates]
       pathInfo = GeneralisedBidirectionalSearch.perform(targetStates)
@@ -319,7 +324,7 @@ class _TransformMultiplexer (_Feature):
     self.availableTransformations.append(self.oopMultiply)
     oopMultiplyID = self.availableTransformations.index(self.oopMultiply)
     
-    for basisPair, path in transformMap.items():
+    for basisPair, path in list(transformMap.items()):
       if any([self.availableTransformations[transformID].get('requiresScaling', False) for transformID, transformPair in path[1::2]]):
         # Scaling is needed.
         # There are different strategies for dealing with this.
