@@ -1,4 +1,4 @@
-#!/usr/bin/env python
+#!/usr/bin/env python3
 # encoding: utf-8
 """
 _Integrator.py
@@ -139,7 +139,7 @@ class _Integrator (_Segment):
   # be calculated in other operators that would have already been evaluated.
   def intraStepOperatorContainersInFieldDescendingOrder(self):
     operatorContainers = self.intraStepOperatorContainers[:]
-    operatorContainers.sort(lambda x, y: cmp(len(x.field.dimensions), len(y.field.dimensions)), reverse=True)
+    operatorContainers.sort(key = lambda x: len(x.field.dimensions), reverse=True)
     return operatorContainers
   
   def ipEvolveFunctionBody(self, function):
