@@ -1,4 +1,4 @@
-#!/usr/bin/env python
+#!/usr/bin/env python3
 # encoding: utf-8
 """
 _ScriptElement.py
@@ -53,7 +53,7 @@ class _ScriptElement (Template):
     to be called again.
     """
     _ScriptElement._callOnceGuards.clear()
-    for instanceGuardSet in _ScriptElement._callOncePerInstanceGuards.itervalues():
+    for instanceGuardSet in _ScriptElement._callOncePerInstanceGuards.values():
       instanceGuardSet.clear()
   
   @property
@@ -125,7 +125,7 @@ class _ScriptElement (Template):
   @property
   def children(self):
     result = self._children[:]
-    result.extend(self.codeBlocks.values())
+    result.extend(list(self.codeBlocks.values()))
     return result
   
   @lazy_property
@@ -169,7 +169,7 @@ class _ScriptElement (Template):
     """
     try:
       getattr(self, attrName)
-    except AttributeError, err:
+    except AttributeError as err:
       if hasattr(type(self), attrName):
         raise
       return False
@@ -196,13 +196,13 @@ class _ScriptElement (Template):
   def functionPrototypes(self):
     if not self.functions:
       return
-    return ''.join([f.prototype() for f in self.functions.itervalues() if f.predicate()])
-  
+    return ''.join([f.prototype() for f in self.functions.values() if f.predicate()])
+
   # Function implemenations
   def functionImplementations(self):
     if not self.functions:
       return
-    return '\n\n'.join([f.implementation() for f in self.functions.itervalues() if f.predicate()])
+    return '\n\n'.join([f.implementation() for f in self.functions.values() if f.predicate()])
   
   # Define a whole bunch of static versions of these functions
   def static_includes(self):
@@ -255,15 +255,15 @@ class _ScriptElement (Template):
     if self.hasattr('bannedFeatures') and self.bannedFeatures:
       # Check if any of the features in the featureList are in the bannedFeatures
       bannedFeatures = self.bannedFeatures
-      featureList = filter(lambda x: x not in bannedFeatures, featureList)
-    
+      featureList = [x for x in featureList if x not in bannedFeatures]
+
     result = []
     filt = self._CHEETAH__currentFilter
     dict['featureList'] = featureList
     
     # Loop over the features that we should include
     for featureName in featureList:
-      if not featureDictionary.has_key(featureName):
+      if featureName not in featureDictionary:
         # If we don't have the feature, then there isn't much we can do
         continue
       
@@ -357,7 +357,7 @@ class _ScriptElement (Template):
     if self.dependenciesEntity:
       self.dependencies.update(self.vectorsFromEntity(self.dependenciesEntity))
     
-    for codeBlock in self.codeBlocks.itervalues():
+    for codeBlock in self.codeBlocks.values():
       # Warning: This means that codeBlocks will have bindNamedVectors() called twice.
       codeBlock.bindNamedVectors()
     
