@@ -1,4 +1,4 @@
-#!/usr/bin/env python
+#!/usr/bin/env python3
 # encoding: utf-8
 """
 parser2.py
@@ -25,6 +25,7 @@ import os
 import sys
 import getopt
 import xpdeint.Python24Support
+import xpdeint.Python3Support
 import xml
 from xml.dom import minidom
 import xpdeint.minidom_extras
@@ -35,7 +36,7 @@ import shutil
 
 DATA_CACHE_VERSION = 2
 
-import cPickle
+import pickle
 
 from xpdeint.Preferences import xpdeintUserDataPath
 
@@ -76,10 +77,11 @@ Options and arguments:
                                   This option is only meaningful when used with --(re)configure
 --lib-path /path/to/lib         : Add the path /path/to/lib to the list of paths searched for libraries
                                   This option is only meaningful when used with --(re)configure
+
 '''
 
 def fileContentsHash(filename):
-  return hashlib.sha1(file(filename).read()).hexdigest()
+  return hashlib.sha1(open(filename).read().encode('utf-8')).hexdigest()
   
 
 def anyObject(iterable):
@@ -98,7 +100,7 @@ def degsOutput(err, globalNameSpace):
   lineNumber = err.lineNumber
   columnNumber = err.columnNumber
   err.msg = '\n' + err.msg + '\n'
-  print >> sys.stderr, err.msg
+  sys.stderr.write(err.msg)
   if not lineNumber == None:
     positionReference =  ["Error caused at line %(lineNumber)i" % locals()]
     if not columnNumber == None:
@@ -107,11 +109,11 @@ def degsOutput(err, globalNameSpace):
     positionReference.append(globalNameSpace['inputScript'].splitlines(True)[lineNumber-1])
     if not columnNumber == None:
         positionReference.append(" "*(columnNumber-1) + "^~~ here.")
-    print >> sys.stderr, ''.join(positionReference) + '\n'
+    sys.stderr.write(''.join(positionReference) + '\n')
   if err.element:
-    print >> sys.stderr, "In element: " + err.element.userUnderstandableXPath()
+    sys.stderr.write("In element: " + err.element.userUnderstandableXPath() + "\n")
   else:
-    print >> sys.stderr, "Unknown element. Please report this error to %s" % globalNameSpace['bugReportAddress']
+    sys.stderr.write("Unknown element. Please report this error to %s" % globalNameSpace['bugReportAddress'])
 
 class Usage(Exception):
   """
@@ -137,12 +139,12 @@ def main(argv=None):
   noVersionInformation = False
   
   # Import version information
-  from Preferences import versionString
-  from Version import subversionRevisionString
+  from .Preferences import versionString
+  from .Version import subversionRevisionString
   
-  print "xmds2 version %(versionString)s (%(subversionRevisionString)s)" % locals()
-  print "Copyright 2000-2014 Graham Dennis, Joseph Hope, Mattias Johnsson"
-  print "                    and the xmds team"
+  print("xmds2 version %(versionString)s (%(subversionRevisionString)s)" % locals())
+  print("Copyright 2000-2019 Graham Dennis, Joseph Hope, Mattias Johnsson")
+  print("                    and the xmds team")
   
   if not os.path.isdir(xpdeintUserDataPath):
       os.mkdir(xpdeintUserDataPath)
@@ -171,7 +173,7 @@ def main(argv=None):
                     ]
       )
       del sys.argv[1:]
-    except getopt.error, msg:
+    except getopt.error as msg:
       raise Usage(msg)
     
     includePaths = []
@@ -222,9 +224,9 @@ def main(argv=None):
     else:
         raise Usage(help_message)
   
-  except Usage, err:
-    print >> sys.stderr, sys.argv[0].split("/")[-1] + ": " + str(err.msg)
-    print >> sys.stderr, "\t for help use --help"
+  except Usage as err:
+    sys.stderr.write(sys.argv[0].split("/")[-1] + ": " + str(err.msg))
+    sys.stderr.write("\nFor help use \"xmds2 --help\"\n")
     return 2
   
   wscript_path = resource_filename(__name__, 'support/wscript')
@@ -234,7 +236,7 @@ def main(argv=None):
   if not os.path.isfile(wscript_userdata_path) or \
     fileContentsHash(wscript_userdata_path) != fileContentsHash(wscript_path) or \
     not os.path.exists(waf_build_cache_path):
-    print "Reconfiguring xmds2 (updated config script)..."
+    print("Reconfiguring xmds2 (updated config script)...")
     
     Configuration.run_reconfig()
   
@@ -248,9 +250,9 @@ def main(argv=None):
   
   # Open the script file
   try:
-    scriptFile = file(scriptName)
-  except Exception, err:
-    print >> sys.stderr, "Exception raised while trying to read xmds script:", err
+    scriptFile = open(scriptName)
+  except Exception as err:
+    sys.stderr.write("Exception raised while trying to read xmds script: " + str(err) + "\n")
     return -1
   
   # Read the contents of the file
@@ -259,18 +261,18 @@ def main(argv=None):
   scriptFile.close()
   del scriptFile
   
-  print "Generating source code..."
+  print("Generating source code...")
   
   # Parse the XML input script into a set of XML
   # classes
   try:
     xmlDocument = minidom.parseString(globalNameSpace['inputScript'])
-  except xml.parsers.expat.ExpatError, err:
-    print >> sys.stderr, "XML Parser error:", err
+  except xml.parsers.expat.ExpatError as err:
+    sys.stderr.write("XML Parser error:" + str(err) +"\n")
     if debug: raise
     return -1
-  except Exception, err:
-    print >> sys.stderr, "Exception raised during parsing xmds script:", err
+  except Exception as err:
+    sys.stderr.write("Exception raised during parsing xmds script: " + str(err) + "\n")
     if debug: raise
     return -1
   
@@ -278,14 +280,14 @@ def main(argv=None):
   # the schema
   try:
     from lxml import etree
-  except ImportError, err:
+  except ImportError as err:
     pass
   else:
     # Parse the schema
     relaxng_doc = etree.parse(resource_filename(__name__, 'support/xpdeint.rng'))
     relaxng = etree.RelaxNG(relaxng_doc)
     # Parse the script
-    script_doc = etree.fromstring(globalNameSpace['inputScript'])
+    script_doc = etree.fromstring(globalNameSpace['inputScript'].encode('ascii'))
     if not relaxng.validate(script_doc):
       # Validation failed
       for error in relaxng.error_log:
@@ -314,14 +316,14 @@ def main(argv=None):
       try:
         import mpmath
         mpmath.mp.prec = 128
-      except ImportError, err:
+      except ImportError as err:
         pass
       dataCacheFile = open(xpdeintDataCachePath, 'rb')
-      dataCache = cPickle.load(dataCacheFile)
+      dataCache = pickle.load(dataCacheFile)
       dataCacheFile.close()
       del dataCacheFile
-    except Exception, err:
-      print >> sys.stderr, "Warning: Unable to load xmds2 data cache."
+    except Exception as err:
+      sys.stderr.write("Warning: Unable to load xmds2 data cache.\n")
       if debug: raise
   globalNameSpace['dataCache'] = dataCache
   
@@ -345,7 +347,7 @@ def main(argv=None):
     
     # We don't have a parser that understands the XML, so we must bail
     if not parser:
-      print >> sys.stderr, "Unable to recognise file as an xmds script of the correct version."
+      sys.stderr.write("\nUnable to recognise file as an xmds script of the correct version. Aborting.\n")
       return -1
     
     # Set our magic filter
@@ -390,14 +392,14 @@ def main(argv=None):
   # If there was an exception during parsing or preflight, a ParserException should have been
   # raised. The ParserException knows the XML element that triggered the exception, and the 
   # error string that should be presented to the user.
-  except ParserException, err: 
+  except ParserException as err:
     if degs:
       degsOutput(err, globalNameSpace)
       return -1
     # Print the error to the user
     lineNumber = err.lineNumber
     columnNumber = err.columnNumber
-    print >> sys.stderr, err.msg
+    sys.stderr.write(err.msg + "\n")
     if not lineNumber == None:
       positionReference =  ["    Error caused at line %(lineNumber)i" % locals()]
       if not columnNumber == None:
@@ -406,12 +408,12 @@ def main(argv=None):
       positionReference.append(globalNameSpace['inputScript'].splitlines(True)[lineNumber-1])
       if not columnNumber == None:
           positionReference.append(" "*(columnNumber-1) + "^~~ here.")
-      print >> sys.stderr, ''.join(positionReference)
+      sys.stderr.write(''.join(positionReference) + "\n")
     if debug:
       if err.element:
-        print >> sys.stderr, "    In element: " + err.element.userUnderstandableXPath()
+        sys.stderr.write("    In element: " + err.element.userUnderstandableXPath() + "\n")
       else:
-        print >> sys.stderr, "    Unknown element. Please report this error to %s" % globalNameSpace['bugReportAddress']
+        sys.stderr.write("    Unknown element. Please report this error to %s" % globalNameSpace['bugReportAddress'] + "\n")
     
     # If we have the debug option on, then in addition to the path to the XML element
     # that triggered the exception, print a traceback showing the list of Python function
@@ -424,12 +426,12 @@ def main(argv=None):
   dataCache = globalNameSpace['dataCache']
   if dataCache:
     try:
-      dataCacheFile = file(xpdeintDataCachePath, 'w')
-    except IOError, err:
-      print >> sys.stderr, "Warning: Unable to write xmds2 data cache. " \
-                           "Ensure '%(xpdeintUserDataPath)s' exists and is writable." % locals()
+      dataCacheFile = open(xpdeintDataCachePath, 'wb')
+    except IOError as err:
+      sys.stderr.write("Warning: Unable to write xmds2 data cache. " \
+                           "Ensure '%(xpdeintUserDataPath)s' exists and is writable." % locals())
     else:
-      cPickle.dump(dataCache, dataCacheFile, protocol = 2)
+      pickle.dump(dataCache, dataCacheFile, protocol = 2)
       dataCacheFile.close()
   
   
@@ -448,9 +450,9 @@ def main(argv=None):
         lines[lineNumber] = line.replace('_XPDEINT_CORRECT_MISSING_LINE_NUMBER_', '%i "%s"' % (lineNumber+2, sourceFilename))
     simulationContents = ''.join(lines)
   
-  file(sourceFilename, 'w').write(simulationContents)
+  open(sourceFilename, 'w').write(simulationContents)
   
-  print "... done"
+  print("... done")
   
   if debug:
     globalNameSpace['simulationUselib'].add('debug')
@@ -475,7 +477,7 @@ def main(argv=None):
   assert len(variant) == 1
   
   if compileScript:
-      print "Compiling simulation..."
+      print("Compiling simulation...")
     
       result = Configuration.run_build(
         sourceFilename,
@@ -487,9 +489,9 @@ def main(argv=None):
       )
       
       if result == 0:
-        print "... done. Type './%s' to run." % globalNameSpace['simulationName']
+        print("... done. Type './%s' to run." % globalNameSpace['simulationName'])
       else:
-        print "\n\nFATAL ERROR: Failed to compile. Check warnings and errors. The most important will be first."
+        print("\n\nFATAL ERROR: Failed to compile. Check warnings and errors. The most important will be first.")
       
       return result
   
