@@ -1,6 +1,6 @@
-#!/usr/bin/env python
+#!/usr/bin/env python3
 # encoding: utf-8
-# Thomas Nagy, 2008-2010 (ita)
+# Thomas Nagy, 2008-2018 (ita)
 
 "Detect as/gas/gcc for compiling assembly files"
 
@@ -11,7 +11,8 @@ def configure(conf):
 	"""
 	Find the programs gas/as/gcc and set the variable *AS*
 	"""
-	conf.find_program(['gas', 'as', 'gcc'], var='AS')
-	conf.env.AS_TGT_F = ['-o']
+	conf.find_program(['gas', 'gcc'], var='AS')
+	conf.env.AS_TGT_F = ['-c', '-o']
 	conf.env.ASLNK_TGT_F = ['-o']
 	conf.find_ar()
+	conf.load('asm')
